package sun.misc;

import java.math.BigInteger;
import java.util.Arrays;

public class FDBigInteger
{
    static final int[] SMALL_5_POW;
    static final long[] LONG_5_POW;
    private static final int MAX_FIVE_POW = 340;
    private static final FDBigInteger[] POW_5_CACHE;
    public static final FDBigInteger ZERO;
    private static final long LONG_MASK = 4294967295L;
    private int[] data;
    private int offset;
    private int nWords;
    private boolean isImmutable;
    
    private FDBigInteger(int[] data, int offset) {
        this.isImmutable = false;
        this.data = data;
        this.offset = offset;
        this.nWords = data.length;
        this.trimLeadingZeros();
    }
    
    public FDBigInteger(long n, char[] array, int i, int n2) {
        this.isImmutable = false;
        (this.data = new int[Math.max((n2 + 8) / 9, 2)])[0] = (int)n;
        this.data[1] = (int)(n >>> 32);
        this.offset = 0;
        this.nWords = 2;
        while (i < n2 - 5) {
            int n3;
            int n4;
            for (n3 = i + 5, n4 = array[i++] - 48; i < n3; n4 = 10 * n4 + array[i++] - 48) {}
            this.multAddMe(100000, n4);
        }
        int n5;
        int n6;
        for (n5 = 1, n6 = 0; i < n2; n6 = 10 * n6 + array[i++] - 48, n5 *= 10) {}
        if (n5 != 1) {
            this.multAddMe(n5, n6);
        }
        this.trimLeadingZeros();
    }
    
    public static FDBigInteger valueOfPow52(int n, int n2) {
        if (n != 0) {
            if (n2 == 0) {
                return big5pow(n);
            }
            if (n < FDBigInteger.SMALL_5_POW.length) {
                int n3 = FDBigInteger.SMALL_5_POW[n];
                int n4 = n2 >> 5;
                int n5 = n2 & 0x1F;
                if (n5 == 0) {
                    return new FDBigInteger(new int[] { n3 }, n4);
                }
                return new FDBigInteger(new int[] { n3 << n5, n3 >>> 32 - n5 }, n4);
            }
            else {
                return big5pow(n).leftShift(n2);
            }
        }
        else {
            return valueOfPow2(n2);
        }
    }
    
    public static FDBigInteger valueOfMulPow52(long n, int detailMessage, int detailMessage2) {
        assert detailMessage >= 0 : detailMessage;
        assert detailMessage2 >= 0 : detailMessage2;
        int n2 = (int)n;
        int n3 = (int)(n >>> 32);
        int n4 = detailMessage2 >> 5;
        int n5 = detailMessage2 & 0x1F;
        if (detailMessage != 0) {
            if (detailMessage < FDBigInteger.SMALL_5_POW.length) {
                long n6 = (long)FDBigInteger.SMALL_5_POW[detailMessage] & 0xFFFFFFFFL;
                long n7 = ((long)n2 & 0xFFFFFFFFL) * n6;
                int n8 = (int)n7;
                long n9 = ((long)n3 & 0xFFFFFFFFL) * n6 + (n7 >>> 32);
                int n10 = (int)n9;
                int n11 = (int)(n9 >>> 32);
                if (n5 == 0) {
                    return new FDBigInteger(new int[] { n8, n10, n11 }, n4);
                }
                return new FDBigInteger(new int[] { n8 << n5, n10 << n5 | n8 >>> 32 - n5, n11 << n5 | n10 >>> 32 - n5, n11 >>> 32 - n5 }, n4);
            }
            else {
                FDBigInteger big5pow = big5pow(detailMessage);
                int[] array;
                if (n3 == 0) {
                    array = new int[big5pow.nWords + 1 + ((detailMessage2 != 0) ? 1 : 0)];
                    mult(big5pow.data, big5pow.nWords, n2, array);
                }
                else {
                    array = new int[big5pow.nWords + 2 + ((detailMessage2 != 0) ? 1 : 0)];
                    mult(big5pow.data, big5pow.nWords, n2, n3, array);
                }
                return new FDBigInteger(array, big5pow.offset).leftShift(detailMessage2);
            }
        }
        else if (detailMessage2 != 0) {
            if (n5 == 0) {
                return new FDBigInteger(new int[] { n2, n3 }, n4);
            }
            return new FDBigInteger(new int[] { n2 << n5, n3 << n5 | n2 >>> 32 - n5, n3 >>> 32 - n5 }, n4);
        }
        else {
            return new FDBigInteger(new int[] { n2, n3 }, 0);
        }
    }
    
    private static FDBigInteger valueOfPow2(int n) {
        return new FDBigInteger(new int[] { 1 << (n & 0x1F) }, n >> 5);
    }
    
    private void trimLeadingZeros() {
        int nWords = this.nWords;
        if (nWords > 0 && this.data[--nWords] == 0) {
            while (nWords > 0 && this.data[nWords - 1] == 0) {
                --nWords;
            }
            if ((this.nWords = nWords) == 0) {
                this.offset = 0;
            }
        }
    }
    
    public int getNormalizationBias() {
        if (this.nWords == 0) {
            throw new IllegalArgumentException("Zero value cannot be normalized");
        }
        int numberOfLeadingZeros = Integer.numberOfLeadingZeros(this.data[this.nWords - 1]);
        return (numberOfLeadingZeros < 4) ? (28 + numberOfLeadingZeros) : (numberOfLeadingZeros - 4);
    }
    
    private static void leftShift(int[] array, int i, int[] array2, int n, int n2, int n3) {
        while (i > 0) {
            int n4 = n3 << n;
            n3 = array[i - 1];
            array2[i] = (n4 | n3 >>> n2);
            --i;
        }
        array2[0] = n3 << n;
    }
    
    public FDBigInteger leftShift(int n) {
        if (n == 0 || this.nWords == 0) {
            return this;
        }
        int n2 = n >> 5;
        int n3 = n & 0x1F;
        if (this.isImmutable) {
            if (n3 == 0) {
                return new FDBigInteger(Arrays.copyOf(this.data, this.nWords), this.offset + n2);
            }
            int n10 = 32 - n3;
            int n11 = this.nWords - 1;
            int n12 = this.data[n11];
            int n13 = n12 >>> n10;
            int[] array;
            if (n13 != 0) {
                array = new int[this.nWords + 1];
                array[this.nWords] = n13;
            }
            else {
                array = new int[this.nWords];
            }
            leftShift(this.data, n11, array, n3, n10, n12);
            return new FDBigInteger(array, this.offset + n2);
        }
        else {
            if (n3 != 0) {
                int n4 = 32 - n3;
                if (this.data[0] << n3 == 0) {
                    int i = 0;
                    int n8 = this.data[i];
                    while (i < this.nWords - 1) {
                        int n9 = n8 >>> n4;
                        n8 = this.data[i + 1];
                        this.data[i] = (n9 | n8 << n3);
                        ++i;
                    }
                    if ((this.data[i] = n8 >>> n4) == 0) {
                        --this.nWords;
                    }
                    ++this.offset;
                }
                else {
                    int n5 = this.nWords - 1;
                    int n6 = this.data[n5];
                    int n7 = n6 >>> n4;
                    int[] data = this.data;
                    int[] data2 = this.data;
                    if (n7 != 0) {
                        if (this.nWords == this.data.length) {
                            data = (this.data = new int[this.nWords + 1]);
                        }
                        data[this.nWords++] = n7;
                    }
                    leftShift(data2, n5, data, n3, n4, n6);
                }
            }
            this.offset += n2;
            return this;
        }
    }
    
    private int size() {
        return this.nWords + this.offset;
    }
    
    public int quoRemIteration(FDBigInteger fdBigInteger) throws IllegalArgumentException {
        assert !this.isImmutable : "cannot modify immutable value";
        int size = this.size();
        int size2 = fdBigInteger.size();
        if (size < size2) {
            int multAndCarryBy11 = multAndCarryBy10(this.data, this.nWords, this.data);
            if (multAndCarryBy11 != 0) {
                this.data[this.nWords++] = multAndCarryBy11;
            }
            else {
                this.trimLeadingZeros();
            }
            return 0;
        }
        else {
            if (size > size2) {
                throw new IllegalArgumentException("disparate values");
            }
            long n = ((long)this.data[this.nWords - 1] & 0xFFFFFFFFL) / ((long)fdBigInteger.data[fdBigInteger.nWords - 1] & 0xFFFFFFFFL);
            if (this.multDiffMe(n, fdBigInteger) != 0L) {
                long detailMessage = 0L;
                int i = fdBigInteger.offset - this.offset;
                int[] data = fdBigInteger.data;
                int[] data2 = this.data;
                while (detailMessage == 0L) {
                    int n2 = 0;
                    for (; i < this.nWords; ++i) {
                        long n3 = detailMessage + (((long)data2[i] & 0xFFFFFFFFL) + ((long)data[n2] & 0xFFFFFFFFL));
                        data2[i] = (int)n3;
                        detailMessage = n3 >>> 32;
                        ++n2;
                    }
                    assert detailMessage == 1L : detailMessage;
                    --n;
                }
            }
            int multAndCarryBy10 = multAndCarryBy10(this.data, this.nWords, this.data);
            assert multAndCarryBy10 == 0 : multAndCarryBy10;
            this.trimLeadingZeros();
            return (int)n;
        }
    }
    
    public FDBigInteger multBy10() {
        if (this.nWords == 0) {
            return this;
        }
        if (this.isImmutable) {
            int[] array = new int[this.nWords + 1];
            array[this.nWords] = multAndCarryBy10(this.data, this.nWords, array);
            return new FDBigInteger(array, this.offset);
        }
        int multAndCarryBy10 = multAndCarryBy10(this.data, this.nWords, this.data);
        if (multAndCarryBy10 != 0) {
            if (this.nWords == this.data.length) {
                if (this.data[0] == 0) {
                    System.arraycopy(this.data, 1, this.data, 0, --this.nWords);
                    ++this.offset;
                }
                else {
                    this.data = Arrays.copyOf(this.data, this.data.length + 1);
                }
            }
            this.data[this.nWords++] = multAndCarryBy10;
            return this;
        }
        else {
            this.trimLeadingZeros();
            return this;
        }
    }
    
    public FDBigInteger multByPow52(int n, int n2) {
        if (this.nWords == 0) {
            return this;
        }
        if (n != 0) {
            int n3 = n2 != 0;
            if (n < FDBigInteger.SMALL_5_POW.length) {
                int[] array2 = new int[this.nWords + 1 + n3];
                mult(this.data, this.nWords, FDBigInteger.SMALL_5_POW[n], array2);
                this = new FDBigInteger(array2, this.offset);
            }
            else {
                FDBigInteger big5pow = big5pow(n);
                int[] array = new int[this.nWords + big5pow.size() + n3];
                mult(this.data, this.nWords, big5pow.data, big5pow.nWords, array);
                this = new FDBigInteger(array, this.offset + big5pow.offset);
            }
        }
        return this.leftShift(n2);
    }
    
    private static void mult(int[] array, int n, int[] array2, int n2, int[] array3) {
        long n4;
        for (int i = 0; i < n; array3[i + n2] = (int)n4, ++i) {
            long n3 = (long)array[i] & 0xFFFFFFFFL;
            n4 = 0L;
            for (int j = 0; j < n2; ++j) {
                long n5 = n4 + (((long)array3[i + j] & 0xFFFFFFFFL) + n3 * ((long)array2[j] & 0xFFFFFFFFL));
                array3[i + j] = (int)n5;
                n4 = n5 >>> 32;
            }
        }
    }
    
    public FDBigInteger leftInplaceSub(FDBigInteger fdBigInteger) {
        assert this.size() >= fdBigInteger.size() : "result should be positive";
        FDBigInteger fdBigInteger2;
        if (this.isImmutable) {
            fdBigInteger2 = new FDBigInteger((int[])this.data.clone(), this.offset);
        }
        else {
            fdBigInteger2 = this;
        }
        int n = fdBigInteger.offset - fdBigInteger2.offset;
        int[] data = fdBigInteger.data;
        int[] data2 = fdBigInteger2.data;
        int nWords = fdBigInteger.nWords;
        int nWords2 = fdBigInteger2.nWords;
        int[] array;
        if (n < 0) {
            int nWords3 = nWords2 - n;
            if (nWords3 < array.length) {
                System.arraycopy(array, 0, array, -n, nWords2);
                Arrays.fill(array, 0, -n, 0);
            }
            else {
                array = new int[nWords3];
                System.arraycopy(array, 0, array, -n, nWords2);
                fdBigInteger2.data = array;
            }
            fdBigInteger2.offset = fdBigInteger.offset;
            nWords2 = (fdBigInteger2.nWords = nWords3);
            n = 0;
        }
        long detailMessage = 0L;
        for (int n2 = 0; n2 < nWords && n < nWords2; ++n2, ++n) {
            long n3 = ((long)array[n] & 0xFFFFFFFFL) - ((long)data[n2] & 0xFFFFFFFFL) + detailMessage;
            array[n] = (int)n3;
            detailMessage = n3 >> 32;
        }
        while (detailMessage != 0L && n < nWords2) {
            long n4 = ((long)array[n] & 0xFFFFFFFFL) + detailMessage;
            array[n] = (int)n4;
            detailMessage = n4 >> 32;
            ++n;
        }
        assert detailMessage == 0L : detailMessage;
        fdBigInteger2.trimLeadingZeros();
        return fdBigInteger2;
    }
    
    public FDBigInteger rightInplaceSub(FDBigInteger fdBigInteger) {
        assert this.size() >= fdBigInteger.size() : "result should be positive";
        if (fdBigInteger.isImmutable) {
            fdBigInteger = new FDBigInteger((int[])fdBigInteger.data.clone(), fdBigInteger.offset);
        }
        int n = this.offset - fdBigInteger.offset;
        int[] data = fdBigInteger.data;
        int[] data2 = this.data;
        int nWords = fdBigInteger.nWords;
        int nWords2 = this.nWords;
        int[] a;
        if (n < 0) {
            if ((nWords2 = nWords2) < a.length) {
                System.arraycopy(a, 0, a, -n, nWords);
                Arrays.fill(a, 0, -n, 0);
            }
            else {
                a = new int[nWords2];
                System.arraycopy(a, 0, a, -n, nWords);
                fdBigInteger.data = a;
            }
            fdBigInteger.offset = this.offset;
            n = 0;
        }
        else {
            int newLength = nWords2 + n;
            if (newLength >= a.length) {
                fdBigInteger.data = Arrays.copyOf(a, newLength);
            }
        }
        int i = 0;
        long detailMessage = 0L;
        while (i < n) {
            long n2 = 0L - ((long)a[i] & 0xFFFFFFFFL) + detailMessage;
            a[i] = (int)n2;
            detailMessage = n2 >> 32;
            ++i;
        }
        for (int j = 0; j < nWords2; ++j) {
            long n3 = ((long)data2[j] & 0xFFFFFFFFL) - ((long)a[i] & 0xFFFFFFFFL) + detailMessage;
            a[i] = (int)n3;
            detailMessage = n3 >> 32;
            ++i;
        }
        assert detailMessage == 0L : detailMessage;
        fdBigInteger.nWords = i;
        fdBigInteger.trimLeadingZeros();
        return fdBigInteger;
    }
    
    private static int checkZeroTail(int[] array, int i) {
        while (i > 0) {
            if (array[--i] == 0) {
                continue;
            }
            return 1;
        }
        return 0;
    }
    
    public int cmp(FDBigInteger fdBigInteger) {
        int n = this.nWords + this.offset;
        int n2 = fdBigInteger.nWords + fdBigInteger.offset;
        if (n > n2) {
            return 1;
        }
        if (n < n2) {
            return -1;
        }
        int nWords = this.nWords;
        int nWords2 = fdBigInteger.nWords;
        while (nWords > 0 && nWords2 > 0) {
            int n3 = this.data[--nWords];
            int n4 = fdBigInteger.data[--nWords2];
            if (n3 != n4) {
                return (((long)n3 & 0xFFFFFFFFL) < ((long)n4 & 0xFFFFFFFFL)) ? -1 : 1;
            }
            continue;
        }
        if (nWords > 0) {
            return checkZeroTail(this.data, nWords);
        }
        if (nWords2 > 0) {
            return -checkZeroTail(fdBigInteger.data, nWords2);
        }
        return 0;
    }
    
    public int cmpPow52(int n, int n2) {
        if (n == 0) {
            int n3 = n2 >> 5;
            int n4 = n2 & 0x1F;
            int n5 = this.nWords + this.offset;
            if (n5 > n3 + 1) {
                return 1;
            }
            if (n5 < n3 + 1) {
                return -1;
            }
            int n6 = this.data[this.nWords - 1];
            int n7 = 1 << n4;
            if (n6 != n7) {
                return (((long)n6 & 0xFFFFFFFFL) < ((long)n7 & 0xFFFFFFFFL)) ? -1 : 1;
            }
            return checkZeroTail(this.data, this.nWords - 1);
        }
        else {
            return this.cmp(big5pow(n).leftShift(n2));
        }
    }
    
    public int addAndCmp(FDBigInteger fdBigInteger, FDBigInteger fdBigInteger2) {
        int size = fdBigInteger.size();
        int size2 = fdBigInteger2.size();
        if (size >= size2) {
        }
        else {
            fdBigInteger = (fdBigInteger2 = fdBigInteger2);
            size = (size2 = size2);
        }
        int size3 = this.size();
        if (size == 0) {
            return size3 != 0?1:0;
        }
        if (size2 == 0) {
            return this.cmp(fdBigInteger);
        }
        if (size > size3) {
            return -1;
        }
        if (size + 1 < size3) {
            return 1;
        }
        long n = (long)fdBigInteger.data[fdBigInteger.nWords - 1] & 0xFFFFFFFFL;
        if (size2 == size) {
            n += ((long)fdBigInteger2.data[fdBigInteger2.nWords - 1] & 0xFFFFFFFFL);
        }
        if (n >>> 32 == 0L) {
            if (n + 1L >>> 32 == 0L) {
                if (size < size3) {
                    return 1;
                }
                long n4 = (long)this.data[this.nWords - 1] & 0xFFFFFFFFL;
                if (n4 < n) {
                    return -1;
                }
                if (n4 > n + 1L) {
                    return 1;
                }
            }
        }
        else {
            if (size + 1 > size3) {
                return -1;
            }
            long n2 = n >>> 32;
            long n3 = (long)this.data[this.nWords - 1] & 0xFFFFFFFFL;
            if (n3 < n2) {
                return -1;
            }
            if (n3 > n2 + 1L) {
                return 1;
            }
        }
        return this.cmp(fdBigInteger.add(fdBigInteger2));
    }
    
    public void makeImmutable() {
        this.isImmutable = true;
    }
    
    private FDBigInteger mult(int n) {
        if (this.nWords == 0) {
            return this;
        }
        int[] array = new int[this.nWords + 1];
        mult(this.data, this.nWords, n, array);
        return new FDBigInteger(array, this.offset);
    }
    
    private FDBigInteger mult(FDBigInteger fdBigInteger) {
        if (this.nWords == 0) {
            return this;
        }
        if (this.size() == 1) {
            return fdBigInteger.mult(this.data[0]);
        }
        if (fdBigInteger.nWords == 0) {
            return fdBigInteger;
        }
        if (fdBigInteger.size() == 1) {
            return this.mult(fdBigInteger.data[0]);
        }
        int[] array = new int[this.nWords + fdBigInteger.nWords];
        mult(this.data, this.nWords, fdBigInteger.data, fdBigInteger.nWords, array);
        return new FDBigInteger(array, this.offset + fdBigInteger.offset);
    }
    
    private FDBigInteger add(FDBigInteger fdBigInteger) {
        int size = this.size();
        int size2 = fdBigInteger.size();
        if (size >= size2) {
        }
        else {
            this = fdBigInteger;
            size = size2;
            fdBigInteger = this;
            size2 = size;
        }
        int[] array = new int[size + 1];
        int i = 0;
        long n = 0L;
        while (i < size2) {
            long n2 = n + (((i < this.offset) ? 0L : ((long)this.data[i - this.offset] & 0xFFFFFFFFL)) + ((i < fdBigInteger.offset) ? 0L : ((long)fdBigInteger.data[i - fdBigInteger.offset] & 0xFFFFFFFFL)));
            array[i] = (int)n2;
            n = n2 >> 32;
            ++i;
        }
        while (i < size) {
            long n3 = n + ((i < this.offset) ? 0L : ((long)this.data[i - this.offset] & 0xFFFFFFFFL));
            array[i] = (int)n3;
            n = n3 >> 32;
            ++i;
        }
        array[size] = (int)n;
        return new FDBigInteger(array, 0);
    }
    
    private void multAddMe(int n, int n2) {
        long n3 = (long)n & 0xFFFFFFFFL;
        long n4 = n3 * ((long)this.data[0] & 0xFFFFFFFFL) + ((long)n2 & 0xFFFFFFFFL);
        this.data[0] = (int)n4;
        long n5 = n4 >>> 32;
        for (int i = 1; i < this.nWords; ++i) {
            long n6 = n5 + n3 * ((long)this.data[i] & 0xFFFFFFFFL);
            this.data[i] = (int)n6;
            n5 = n6 >>> 32;
        }
        if (n5 != 0L) {
            this.data[this.nWords++] = (int)n5;
        }
    }
    
    private long multDiffMe(long n, FDBigInteger fdBigInteger) {
        long n2 = 0L;
        if (n != 0L) {
            int n3 = fdBigInteger.offset - this.offset;
            if (n3 >= 0) {
                int[] data4 = fdBigInteger.data;
                int[] data5 = this.data;
                int j;
                for (j = 0, ; j < fdBigInteger.nWords; ++j, ++n3) {
                    long n9 = n2 + (((long)data5[n3] & 0xFFFFFFFFL) - n * ((long)data4[j] & 0xFFFFFFFFL));
                    data5[n3] = (int)n9;
                    n2 = n9 >> 32;
                }
            }
            else {
                int n4 = -n3;
                int[] data = new int[this.nWords + n4];
                int i = 0;
                int n5 = 0;
                int[] data2 = fdBigInteger.data;
                while (n5 < n4 && i < fdBigInteger.nWords) {
                    long n6 = n2 - n * ((long)data2[i] & 0xFFFFFFFFL);
                    data[n5] = (int)n6;
                    n2 = n6 >> 32;
                    ++i;
                    ++n5;
                }
                int n7 = 0;
                int[] data3 = this.data;
                while (i < fdBigInteger.nWords) {
                    long n8 = n2 + (((long)data3[n7] & 0xFFFFFFFFL) - n * ((long)data2[i] & 0xFFFFFFFFL));
                    data[n5] = (int)n8;
                    n2 = n8 >> 32;
                    ++i;
                    ++n7;
                    ++n5;
                }
                this.nWords += n4;
                this.offset -= n4;
                this.data = data;
            }
        }
        return n2;
    }
    
    private static int multAndCarryBy10(int[] array, int n, int[] array2) {
        long n2 = 0L;
        for (int i = 0; i < n; ++i) {
            long n3 = ((long)array[i] & 0xFFFFFFFFL) * 10L + n2;
            array2[i] = (int)n3;
            n2 = n3 >>> 32;
        }
        return (int)n2;
    }
    
    private static void mult(int[] array, int n, int n2, int[] array2) {
        long n3 = (long)n2 & 0xFFFFFFFFL;
        long n4 = 0L;
        for (int i = 0; i < n; ++i) {
            long n5 = ((long)array[i] & 0xFFFFFFFFL) * n3 + n4;
            array2[i] = (int)n5;
            n4 = n5 >>> 32;
        }
        array2[n] = (int)n4;
    }
    
    private static void mult(int[] array, int n, int n2, int n3, int[] array2) {
        long n4 = (long)n2 & 0xFFFFFFFFL;
        long n5 = 0L;
        for (int i = 0; i < n; ++i) {
            long n6 = n4 * ((long)array[i] & 0xFFFFFFFFL) + n5;
            array2[i] = (int)n6;
            n5 = n6 >>> 32;
        }
        array2[n] = (int)n5;
        long n7 = (long)n3 & 0xFFFFFFFFL;
        long n8 = 0L;
        for (int j = 0; j < n; ++j) {
            long n9 = ((long)array2[j + 1] & 0xFFFFFFFFL) + n7 * ((long)array[j] & 0xFFFFFFFFL) + n8;
            array2[j + 1] = (int)n9;
            n8 = n9 >>> 32;
        }
        array2[n + 1] = (int)n8;
    }
    
    private static FDBigInteger big5pow(int detailMessage) {
        assert detailMessage >= 0 : detailMessage;
        if (detailMessage < 340) {
            return FDBigInteger.POW_5_CACHE[detailMessage];
        }
        return big5powRec(detailMessage);
    }
    
    private static FDBigInteger big5powRec(int n) {
        if (n < 340) {
            return FDBigInteger.POW_5_CACHE[n];
        }
        int n2 = n >> 1;
        int n3 = n - n2;
        FDBigInteger big5powRec = big5powRec(n2);
        if (n3 < FDBigInteger.SMALL_5_POW.length) {
            return big5powRec.mult(FDBigInteger.SMALL_5_POW[n3]);
        }
        return big5powRec.mult(big5powRec(n3));
    }
    
    public String toHexString() {
        if (this.nWords == 0) {
            return "0";
        }
        StringBuilder sb = new StringBuilder((this.nWords + this.offset) * 8);
        for (int i = this.nWords - 1; i >= 0; --i) {
            String hexString = Integer.toHexString(this.data[i]);
            for (int j = hexString.length(); j < 8; ++j) {
                sb.append('0');
            }
            sb.append(hexString);
        }
        for (int k = this.offset; k > 0; --k) {
            sb.append("00000000");
        }
        return sb.toString();
    }
    
    public BigInteger toBigInteger() {
        byte[] val = new byte[this.nWords * 4 + 1];
        for (int i = 0; i < this.nWords; ++i) {
            int n = this.data[i];
            val[val.length - 4 * i - 1] = (byte)n;
            val[val.length - 4 * i - 2] = (byte)(n >> 8);
            val[val.length - 4 * i - 3] = (byte)(n >> 16);
            val[val.length - 4 * i - 4] = (byte)(n >> 24);
        }
        return new BigInteger(val).shiftLeft(this.offset * 32);
    }
    
    @Override
    public String toString() {
        return this.toBigInteger().toString();
    }
    
    static {
        SMALL_5_POW = new int[] { 1, 5, 25, 125, 625, 3125, 15625, 78125, 390625, 1953125, 9765625, 48828125, 244140625, 1220703125 };
        LONG_5_POW = new long[] { 1L, 5L, 25L, 125L, 625L, 3125L, 15625L, 78125L, 390625L, 1953125L, 9765625L, 48828125L, 244140625L, 1220703125L, 6103515625L, 30517578125L, 152587890625L, 762939453125L, 3814697265625L, 19073486328125L, 95367431640625L, 476837158203125L, 2384185791015625L, 11920928955078125L, 59604644775390625L, 298023223876953125L, 1490116119384765625L };
        POW_5_CACHE = new FDBigInteger[340];
        int i;
        Object o;
        for (i = 0; i < FDBigInteger.SMALL_5_POW.length; FDBigInteger.POW_5_CACHE[i] = o, ++i) {
            o = new FDBigInteger(new int[] { FDBigInteger.SMALL_5_POW[i] }, 0);
            o.makeImmutable();
        }
        FDBigInteger fdBigInteger = FDBigInteger.POW_5_CACHE[i - 1];
        while (i < 340) {
            fdBigInteger = (FDBigInteger.POW_5_CACHE[i] = fdBigInteger.mult(5));
            fdBigInteger.makeImmutable();
            ++i;
        }
        (ZERO = new FDBigInteger(new int[0], 0)).makeImmutable();
    }
}
